@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -25,6 +25,9 @@
 #include "gog-1.5d.h"
 #include "gog-line.h"
 #include "gog-barcol.h"
+#include "gog-dropbar.h"
+#include "gog-minmax.h"
+#include <goffice/graph/gog-series-lines.h>
 #include <goffice/graph/gog-view.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-axis.h>
@@ -190,6 +193,15 @@ gog_plot1_5d_update (GogObject *obj)
 			else
 				go_data_vector_get_minmax (GO_DATA_VECTOR (
 					series->base.values[1].data), &minima, &maxima);
+			if (series->base.plot->desc.series.num_dim == 3) {
+				double tmp_min, tmp_max;
+				go_data_vector_get_minmax (GO_DATA_VECTOR (
+					series->base.values[2].data), &tmp_min, &tmp_max);
+				if (minima > tmp_min )
+					minima = tmp_min;
+				if (maxima < tmp_max)
+					maxima = tmp_max;
+			}
 			if (model->minima > minima)
 				model->minima = minima;
 			if (model->maxima < maxima)
@@ -229,11 +241,14 @@ gog_plot1_5d_update (GogObject *obj)
 			vals[i] = go_data_vector_get_values (
 				GO_DATA_VECTOR (series->base.values[1].data));
 			g_object_get (G_OBJECT (series), "errors", errors + i, NULL);
+			if (errors[i])
+				g_object_unref (errors[i]);
 			lengths[i] = go_data_vector_get_len (
 				GO_DATA_VECTOR (series->base.values[1].data));
 		}
 
-		klass->update_stacked_and_percentage (model, vals, errors, lengths);
+		if (klass->update_stacked_and_percentage)
+			klass->update_stacked_and_percentage (model, vals, errors, lengths);
 	}
 
 	if (old_minima != model->minima || old_maxima != model->maxima)
@@ -268,9 +283,9 @@ gog_plot1_5d_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 	} else if (axis == gog_axis_get_atype (gog_plot1_5d_get_index_axis (model))) {
 		GSList *ptr;
 
-		bounds->val.minima = 0.;
-		bounds->val.maxima = model->num_elements - 1.;
-		bounds->logical.minima = 0.;
+		bounds->val.minima = 1.;
+		bounds->val.maxima = model->num_elements;
+		bounds->logical.minima = 1.;
 		bounds->logical.maxima = go_nan;
 		bounds->is_discrete    = TRUE;
 
@@ -284,24 +299,6 @@ gog_plot1_5d_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 	return NULL;
 }
 
-static GogAxisSet
-gog_plot1_5d_axis_set_pref (GogPlot const *plot)
-{
-	return GOG_AXIS_SET_XY; /* do some magic later for 3d */
-}
-
-static gboolean
-gog_plot1_5d_axis_set_is_valid (GogPlot const *plot, GogAxisSet type)
-{
-	return type == GOG_AXIS_SET_XY; /* do some magic later for 3d */
-}
-
-static gboolean
-gog_plot1_5d_axis_set_assign (GogPlot *plot, GogAxisSet type)
-{
-	return type == GOG_AXIS_SET_XY; /* do some magic later for 3d */
-}
-
 static gboolean
 gog_1_5d_supports_vary_style_by_element (GogPlot const *plot)
 {
@@ -349,10 +346,8 @@ gog_plot1_5d_class_init (GogPlotClass *plot_klass)
 	plot_klass->desc.num_series_min = 1;
 	plot_klass->desc.num_series_max = G_MAXINT;
 	plot_klass->series_type = gog_series1_5d_get_type ();
-	plot_klass->axis_get_bounds   = gog_plot1_5d_axis_get_bounds;
-	plot_klass->axis_set_pref     = gog_plot1_5d_axis_set_pref;
-	plot_klass->axis_set_is_valid = gog_plot1_5d_axis_set_is_valid;
-	plot_klass->axis_set_assign   = gog_plot1_5d_axis_set_assign;
+	plot_klass->axis_get_bounds   	= gog_plot1_5d_axis_get_bounds;
+	plot_klass->axis_set	      	= GOG_AXIS_SET_XY;
 	plot_klass->supports_vary_style_by_element = gog_1_5d_supports_vary_style_by_element;
 }
 
@@ -361,11 +356,94 @@ gog_plot1_5d_init (GogPlot1_5d *plot)
 {
 	plot->fmt = NULL;
 	plot->in_3d = FALSE;
+	plot->support_series_lines = FALSE;
+	plot->support_drop_lines = FALSE;
+	plot->support_lines = FALSE;
 }
 
-GSF_CLASS_ABSTRACT (GogPlot1_5d, gog_plot1_5d,
-		    gog_plot1_5d_class_init, gog_plot1_5d_init,
-		    GOG_PLOT_TYPE)
+GSF_DYNAMIC_CLASS_ABSTRACT (GogPlot1_5d, gog_plot1_5d,
+	gog_plot1_5d_class_init, gog_plot1_5d_init,
+	GOG_PLOT_TYPE)
+
+/*****************************************************************************/
+
+static gboolean
+series_lines_can_add (GogObject const *parent)
+{
+	GogSeries1_5d *series = GOG_SERIES1_5D (parent);
+	GogPlot1_5d *plot = GOG_PLOT1_5D (series->base.plot);
+	/* series lines are supported to implement excel series lines in barcol
+	plots and lines with dropbars and high-low lines */
+	if (GOG_IS_PLOT_BARCOL (plot) && plot->type == GOG_1_5D_NORMAL)
+		return FALSE;
+	return (plot->support_series_lines &&
+								!series->has_series_lines);
+}
+
+static void
+series_lines_post_add (GogObject *parent, GogObject *child)
+{
+	GogSeries1_5d *series = GOG_SERIES1_5D (parent);
+	series->has_series_lines = TRUE;
+	gog_object_request_update (child);
+}
+
+static void
+series_lines_pre_remove (GogObject *parent, GogObject *child)
+{
+	GogSeries1_5d *series = GOG_SERIES1_5D (parent);
+	series->has_series_lines = FALSE;
+}
+
+/*****************************************************************************/
+
+static gboolean
+drop_lines_can_add (GogObject const *parent)
+{
+	GogSeries1_5d *series = GOG_SERIES1_5D (parent);
+	return (GOG_PLOT1_5D (series->base.plot)->support_drop_lines &&
+								!series->has_drop_lines);
+}
+
+static void
+drop_lines_post_add (GogObject *parent, GogObject *child)
+{
+	GogSeries1_5d *series = GOG_SERIES1_5D (parent);
+	series->has_drop_lines = TRUE;
+	gog_object_request_update (child);
+}
+
+static void
+drop_lines_pre_remove (GogObject *parent, GogObject *child)
+{
+	GogSeries1_5d *series = GOG_SERIES1_5D (parent);
+	series->has_drop_lines = FALSE;
+}
+
+/*****************************************************************************/
+
+static gboolean
+lines_can_add (GogObject const *parent)
+{
+	GogSeries1_5d *series = GOG_SERIES1_5D (parent);
+	return (GOG_PLOT1_5D (series->base.plot)->support_lines &&
+		!series->has_lines);
+}
+
+static void
+lines_post_add (GogObject *parent, GogObject *child)
+{
+	GogSeries1_5d *series = GOG_SERIES1_5D (parent);
+	series->has_lines = TRUE;
+	gog_object_request_update (child);
+}
+
+static void
+lines_pre_remove (GogObject *parent, GogObject *child)
+{
+	GogSeries1_5d *series = GOG_SERIES1_5D (parent);
+	series->has_lines = FALSE;
+}
 
 /*****************************************************************************/
 
@@ -398,6 +476,17 @@ gog_series1_5d_update (GogObject *obj)
 	}
 	series->base.num_elements = len;
 
+	if (series->base.plot->desc.series.num_dim == 3) {
+		int tmp = 0;
+		if (series->base.values[2].data != NULL) {
+			vals = go_data_vector_get_values (GO_DATA_VECTOR (series->base.values[2].data));
+			tmp = go_data_vector_get_len 
+				(GO_DATA_VECTOR (series->base.values[2].data));
+		}
+		if (tmp < len)
+			len = tmp;
+	}
+
 	/* queue plot for redraw */
 	gog_object_request_update (GOG_OBJECT (series->base.plot));
 	if (old_num != series->base.num_elements)
@@ -461,6 +550,9 @@ gog_series1_5d_populate_editor (GogObject *obj,
 
 	(GOG_OBJECT_CLASS (gog_series1_5d_parent_klass)->populate_editor) (obj, editor, dalloc, cc);
 
+	if (series->plot->desc.series.num_dim == 3)
+		return;
+
 	if (g_object_class_find_property (G_OBJECT_GET_CLASS (series->plot), "horizontal") == NULL)
 		horizontal = FALSE;
 	else
@@ -469,9 +561,46 @@ gog_series1_5d_populate_editor (GogObject *obj,
 	gog_editor_add_page (editor, error_page, _("Error bars"));
 }
 
+static void
+gog_series1_5d_finalize (GObject *obj)
+{
+	GogSeries1_5d *series = GOG_SERIES1_5D (obj);
+	if (series->errors) {
+		g_object_unref (series->errors);
+		series->errors = NULL;
+	}
+	G_OBJECT_CLASS (gog_series1_5d_parent_klass)->finalize (obj);
+}
+
 static void
 gog_series1_5d_class_init (GogObjectClass *obj_klass)
 {
+	static GogObjectRole const roles[] = {
+		{ N_("Series lines"), "GogSeriesLines",	0,
+			GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+			series_lines_can_add,
+			NULL,
+			NULL,
+			series_lines_post_add,
+			series_lines_pre_remove, NULL },
+		{ N_("Drop lines"), "GogSeriesLines",	1,
+			GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+			drop_lines_can_add,
+			NULL,
+			NULL,
+			drop_lines_post_add,
+			drop_lines_pre_remove,
+			NULL },
+		{ N_("Lines"), "GogSeriesLines",	1,
+			GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+			lines_can_add,
+			NULL,
+			NULL,
+			lines_post_add,
+			lines_pre_remove,
+			NULL },
+	};
+
 	GObjectClass *gobject_klass = (GObjectClass *) obj_klass;
 	GogSeriesClass *gog_series_klass = (GogSeriesClass*) obj_klass;
 
@@ -479,10 +608,13 @@ gog_series1_5d_class_init (GogObjectClass *obj_klass)
 
 	gobject_klass->set_property = gog_series1_5d_set_property;
 	gobject_klass->get_property = gog_series1_5d_get_property;
+	gobject_klass->finalize 	      = gog_series1_5d_finalize;
 	obj_klass->update 	      = gog_series1_5d_update;
 	obj_klass->populate_editor    = gog_series1_5d_populate_editor;
 	gog_series_klass->dim_changed = gog_series1_5d_dim_changed;
 
+	gog_object_register_roles (obj_klass, roles, G_N_ELEMENTS (roles));
+
 	g_object_class_install_property (gobject_klass, SERIES_PROP_ERRORS,
 		g_param_spec_object ("errors", "errors",
 			"GogErrorBar *",
@@ -496,21 +628,34 @@ gog_series1_5d_init (GObject *obj)
 
 	series->errors = NULL;
 	series->index_changed = FALSE;
+	series->has_series_lines = FALSE;
+	series->has_drop_lines = FALSE;
+	series->has_lines = FALSE;
 }
 
-GSF_CLASS (GogSeries1_5d, gog_series1_5d,
-	   gog_series1_5d_class_init, gog_series1_5d_init,
-	   GOG_SERIES_TYPE)
+GSF_DYNAMIC_CLASS (GogSeries1_5d, gog_series1_5d,
+	gog_series1_5d_class_init, gog_series1_5d_init,
+	GOG_SERIES_TYPE)
 
 /* Plugin initialization */
 
 G_MODULE_EXPORT void
 go_plugin_init (GOPlugin *plugin, GOCmdContext *cc)
 {
-	gog_plot1_5d_get_type ();
-	gog_line_plot_get_type ();
-	gog_area_plot_get_type ();
-	gog_barcol_plot_get_type ();
+	GTypeModule *module = go_plugin_get_type_module (plugin);
+	gog_plot1_5d_register_type (module);
+	gog_series1_5d_register_type (module);
+	gog_barcol_plot_register_type (module);
+	gog_barcol_view_register_type (module);
+	gog_dropbar_plot_register_type (module);
+	gog_dropbar_view_register_type (module);
+	gog_line_series_register_type (module);
+	gog_line_plot_register_type (module);
+	gog_area_plot_register_type (module);
+	gog_line_view_register_type (module);
+	gog_minmax_series_register_type (module);
+	gog_minmax_plot_register_type (module);
+	gog_minmax_view_register_type (module);
 }
 
 G_MODULE_EXPORT void
